<?php

  $fontpath = '/usr/share/fonts/truetype/';
  $size = 70;
  $chars = [ "0", "1", "2", "3",
             "4", "5", "6", "7",
             "8", "9", "a", "b",
             "c", "d", "e", "f",
             NULL ];
  $x = 128;
  $y = 128;

  $ignore = [ "armenian",
              "ethiopic",
              "georgian",
              "hebrew",
              "japanese",
              "kacst",
              "kedage",
              "lklug",
              "opens",
              "naskh",
              "thai",
              "lohit",
              "malige",
              "pothana",
              "saab",
              "utkal",
              "vemana"
            ];

  // Create the image
  function imageCreateTransparent($x, $y) {
    $imageOut = imagecreatetruecolor($x, $y);
    $backgroundColor = imagecolorallocatealpha($imageOut, 0, 0, 0, 127);
    imagefill($imageOut, 0, 0, $backgroundColor);
    return $imageOut;
  }

  // Center the text
  function ImageTTFCenter($image, $text, $font, $size, $angle = 45) {
    $xi = imagesx($image);
    $yi = imagesy($image);

    $box = imagettfbbox($size, $angle, $font, $text);

    $xr = abs(max($box[2], $box[4]));
    $yr = abs(max($box[5], $box[7]));

    $x = intval(($xi - $xr) / 2);
    $y = intval(($yi + $yr) / 2);

    return array($x, $y);
  }

  // Name the file
  function ImageName($counter, $text) {

    $filepath = "./" . $text;
    if(!is_dir($filepath)) {
      mkdir($filepath, 0777);
    }
    if(!is_dir($filepath)) {
      echo "Error: failed to create directory \"$filepath\"\n";
      exit(1);
    }

    $filename = sprintf("%04d", $counter);
    $filename = $filepath . "/" . $text . $filename . ".png";

    return $filename;
  }


  $counter = 0;
  $dirwalk = new RecursiveDirectoryIterator($fontpath);
  $extension = Array ( 'ttf', 'truetypefont' );
  foreach(new RecursiveIteratorIterator($dirwalk) as $font) {
    $skip = 0;
    foreach ($ignore as $value) {
      if(stripos(basename($font), $value) !== false) {
        $skip = 1;
        break;
      }
    }
    if($skip) {
     echo "skipped " . $font . "\n";
     continue;
    }
    if(!in_array(strtolower(array_pop(explode('.', $font))), $extension))
      continue;
    echo "Creating trainer images for font # " . $counter . " : " . basename($font) . "\n";
    $counter++;

    for ($i = 0; $chars[$i] != NULL; $i++) {
      $text = $chars[$i];
      $filename = ImageName($counter, $text);
      // echo "  $filename\n";
      $image = imageCreateTransparent($x, $y);

      // Create some colors
      $white = imagecolorallocate($image, 255, 255, 255);
      $grey = imagecolorallocate($image, 128, 128, 128);
      $black = imagecolorallocate($image, 0, 0, 0);

      imagefilledrectangle($image, 0, 0, $x, $y, $white);

      // Add the text
      // imagettftext($image, 20, 0, 10, 20, $black, $font, $text);
      list($fx, $fy) = ImageTTFCenter($image, $text, $font, $size, 0);
      imagettftext($image, $size, 0, $fx, $fy, $black, $font, $text);

      // Save the image
      imagesavealpha($image, true);
      imagepng($image, $filename);
      imagedestroy($image);
    }
  }
?>
